@@ -56,6 +56,7 @@ module Test.QuickCheck
   , quickCheckAll
   , verboseCheckAll
   , forAllProperties
+  , allProperties
     -- ** Testing polymorphic properties
   , polyQuickCheck
   , polyVerboseCheck
@@ -106,6 +107,9 @@ module Test.QuickCheck
   , shrink2
 
     -- ** Helper functions for implementing arbitrary
+  , applyArbitrary2
+  , applyArbitrary3
+  , applyArbitrary4
   , arbitrarySizedIntegral
   , arbitrarySizedNatural
   , arbitrarySizedFractional
@@ -142,6 +146,7 @@ module Test.QuickCheck
   , Fixed(..)
   , OrderedList(..)
   , NonEmptyList(..)
+  , InfiniteList(..)
   , Positive(..)
   , NonZero(..)
   , NonNegative(..)
@@ -158,7 +163,7 @@ module Test.QuickCheck
   , PrintableString(..)
 
     -- ** Functions
-  , Fun
+  , Fun (..)
   , applyFun
   , applyFun2
   , applyFun3
