@@ -1,15 +1,25 @@
 -- | Type classes for random generation of values.
 {-# LANGUAGE CPP #-}
+{-# LANGUAGE FlexibleContexts #-}
 #ifndef NO_GENERICS
 {-# LANGUAGE DefaultSignatures, FlexibleContexts, TypeOperators #-}
 {-# LANGUAGE FlexibleInstances, KindSignatures, ScopedTypeVariables #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
-#if __GLASGOW_HASKELL__ < 710
+#if __GLASGOW_HASKELL__ >= 710
+#define OVERLAPPING_ {-# OVERLAPPING #-}
+#else
 {-# LANGUAGE OverlappingInstances  #-}
+#define OVERLAPPING_
 #endif
 #endif
+#ifndef NO_POLYKINDS
+{-# LANGUAGE PolyKinds #-}
+#endif
 #ifndef NO_SAFE_HASKELL
-{-# LANGUAGE Safe #-}
+{-# LANGUAGE Trustworthy #-}
+#endif
+#ifndef NO_NEWTYPE_DERIVING
+{-# LANGUAGE GeneralizedNewtypeDeriving #-}
 #endif
 module Test.QuickCheck.Arbitrary
   (
@@ -17,7 +27,18 @@ module Test.QuickCheck.Arbitrary
     Arbitrary(..)
   , CoArbitrary(..)
 
+  -- ** Unary and Binary classes
+  , Arbitrary1(..)
+  , arbitrary1
+  , shrink1
+  , Arbitrary2(..)
+  , arbitrary2
+  , shrink2
+
   -- ** Helper functions for implementing arbitrary
+  , applyArbitrary2
+  , applyArbitrary3
+  , applyArbitrary4
   , arbitrarySizedIntegral        -- :: Integral a => Gen a
   , arbitrarySizedNatural         -- :: Integral a => Gen a
   , arbitraryBoundedIntegral      -- :: (Bounded a, Integral a) => Gen a
@@ -25,6 +46,10 @@ module Test.QuickCheck.Arbitrary
   , arbitrarySizedFractional      -- :: Fractional a => Gen a
   , arbitraryBoundedRandom        -- :: (Bounded a, Random a) => Gen a
   , arbitraryBoundedEnum          -- :: (Bounded a, Enum a) => Gen a
+  -- ** Generators for various kinds of character
+  , arbitraryUnicodeChar   -- :: Gen Char
+  , arbitraryASCIIChar     -- :: Gen Char
+  , arbitraryPrintableChar -- :: Gen Char
   -- ** Helper functions for implementing shrink
 #ifndef NO_GENERICS
   , genericShrink      -- :: (Generic a, Arbitrary a, RecursivelyShrink (Rep a), GSubterms (Rep a) a) => a -> [a]
@@ -34,6 +59,8 @@ module Test.QuickCheck.Arbitrary
 #endif
   , shrinkNothing            -- :: a -> [a]
   , shrinkList               -- :: (a -> [a]) -> [a] -> [[a]]
+  , shrinkMap                -- :: Arbitrary a -> (a -> b) -> (b -> a) -> b -> [b]
+  , shrinkMapBy              -- :: (a -> b) -> (b -> a) -> (a -> [a]) -> b -> [b]
   , shrinkIntegral           -- :: Integral a => a -> [a]
   , shrinkRealFrac           -- :: RealFrac a => a -> [a]
   -- ** Helper functions for implementing coarbitrary
@@ -57,6 +84,7 @@ import Control.Applicative
 import Data.Foldable(toList)
 import System.Random(Random)
 import Test.QuickCheck.Gen
+import Test.QuickCheck.Random
 import Test.QuickCheck.Gen.Unsafe
 
 {-
@@ -68,13 +96,15 @@ import Data.Generics
 -}
 
 import Data.Char
-  ( chr
-  , ord
+  ( ord
   , isLower
   , isUpper
   , toLower
   , isDigit
   , isSpace
+  , isPrint
+  , generalCategory
+  , GeneralCategory(..)
   )
 
 #ifndef NO_FIXED
@@ -84,10 +114,6 @@ import Data.Fixed
   )
 #endif
 
-#ifndef NO_NATURALS
-import Numeric.Natural
-#endif
-
 import Data.Ratio
   ( Ratio
   , (%)
@@ -103,11 +129,6 @@ import Data.List
   , nub
   )
 
-#ifndef NO_NONEMPTY
-import Data.List.NonEmpty (NonEmpty (..), nonEmpty)
-import Data.Maybe (mapMaybe)
-#endif
-
 import Data.Version (Version (..))
 
 import Control.Monad
@@ -120,6 +141,8 @@ import Control.Monad
 
 import Data.Int(Int8, Int16, Int32, Int64)
 import Data.Word(Word, Word8, Word16, Word32, Word64)
+import System.Exit (ExitCode(..))
+import Foreign.C.Types
 
 #ifndef NO_GENERICS
 import GHC.Generics
@@ -136,25 +159,53 @@ import qualified Data.Monoid as Monoid
 #ifndef NO_TRANSFORMERS
 import Data.Functor.Identity
 import Data.Functor.Constant
+import Data.Functor.Compose
+import Data.Functor.Product
 #endif
 
 --------------------------------------------------------------------------
 -- ** class Arbitrary
 
 -- | Random generation and shrinking of values.
+--
+-- QuickCheck provides @Arbitrary@ instances for most types in @base@,
+-- except those which incur extra dependencies.
+-- For a wider range of @Arbitrary@ instances see the
+-- <http://hackage.haskell.org/package/quickcheck-instances quickcheck-instances>
+-- package.
 class Arbitrary a where
   -- | A generator for values of the given type.
+  --
+  -- It is worth spending time thinking about what sort of test data
+  -- you want - good generators are often the difference between
+  -- finding bugs and not finding them. You can use 'sample',
+  -- 'label' and 'classify' to check the quality of your test data.
+  --
+  -- There is no generic @arbitrary@ implementation included because we don't
+  -- know how to make a high-quality one. If you want one, consider using the
+  -- <http://hackage.haskell.org/package/testing-feat testing-feat> or
+  -- <http://hackage.haskell.org/package/generic-random generic-random> packages.
+  --
+  -- The <http://www.cse.chalmers.se/~rjmh/QuickCheck/manual.html QuickCheck manual>
+  -- goes into detail on how to write good generators. Make sure to look at it,
+  -- especially if your type is recursive!
   arbitrary :: Gen a
 
   -- | Produces a (possibly) empty list of all the possible
-  -- immediate shrinks of the given value. The default implementation
-  -- returns the empty list, so will not try to shrink the value.
+  -- immediate shrinks of the given value.
+  --
+  -- The default implementation returns the empty list, so will not try to
+  -- shrink the value. If your data type has no special invariants, you can
+  -- enable shrinking by defining @shrink = 'genericShrink'@, but by customising
+  -- the behaviour of @shrink@ you can often get simpler counterexamples.
   --
   -- Most implementations of 'shrink' should try at least three things:
   --
   -- 1. Shrink a term to any of its immediate subterms.
+  --    You can use 'subterms' to do this.
   --
   -- 2. Recursively apply 'shrink' to all immediate subterms.
+  --    You can use 'recursivelyShrink' to do this.
   --
   -- 3. Type-specific shrinkings such as replacing a constructor by a
   --    simpler constructor.
@@ -188,8 +239,7 @@ class Arbitrary a where
   --   the three is fully shrunk.
   --
   -- There is a fair bit of boilerplate in the code above.
-  -- We can avoid it with the help of some generic functions;
-  -- note that these only work on GHC 7.2 and above.
+  -- We can avoid it with the help of some generic functions.
   -- The function 'genericShrink' tries shrinking a term to all of its
   -- subterms and, failing that, recursively shrinks the subterms.
   -- Using it, we can define 'shrink' as:
@@ -214,6 +264,30 @@ class Arbitrary a where
   shrink :: a -> [a]
   shrink _ = []
 
+-- | Lifting of the 'Arbitrary' class to unary type constructors.
+class Arbitrary1 f where
+  liftArbitrary :: Gen a -> Gen (f a)
+  liftShrink    :: (a -> [a]) -> f a -> [f a]
+  liftShrink _ _ = []
+
+arbitrary1 :: (Arbitrary1 f, Arbitrary a) => Gen (f a)
+arbitrary1 = liftArbitrary arbitrary
+
+shrink1 :: (Arbitrary1 f, Arbitrary a) => f a -> [f a]
+shrink1 = liftShrink shrink
+
+-- | Lifting of the 'Arbitrary' class to binary type constructors.
+class Arbitrary2 f where
+  liftArbitrary2 :: Gen a -> Gen b -> Gen (f a b)
+  liftShrink2    :: (a -> [a]) -> (b -> [b]) -> f a b -> [f a b]
+  liftShrink2 _ _ _ = []
+
+arbitrary2 :: (Arbitrary2 f, Arbitrary a, Arbitrary b) => Gen (f a b)
+arbitrary2 = liftArbitrary2 arbitrary arbitrary
+
+shrink2 :: (Arbitrary2 f, Arbitrary a, Arbitrary b) => f a b -> [f a b]
+shrink2 = liftShrink2 shrink shrink
+
 #ifndef NO_GENERICS
 -- | Shrink a term to any of its immediate subterms,
 -- and also recursively shrink all subterms.
@@ -314,18 +388,21 @@ instance GSubtermsIncl f a => GSubtermsIncl (M1 i c f) a where
   gSubtermsIncl (M1 x) = gSubtermsIncl x
 
 -- This is the important case: We've found a term of the same type.
-instance {-# OVERLAPPING #-} GSubtermsIncl (K1 i a) a where
+instance OVERLAPPING_ GSubtermsIncl (K1 i a) a where
   gSubtermsIncl (K1 x) = [x]
 
-instance {-# OVERLAPPING #-} GSubtermsIncl (K1 i a) b where
+instance OVERLAPPING_ GSubtermsIncl (K1 i a) b where
   gSubtermsIncl (K1 _) = []
 
 #endif
 
 -- instances
 
+instance (CoArbitrary a) => Arbitrary1 ((->) a) where
+  liftArbitrary arbB = promote (`coarbitrary` arbB)
+
 instance (CoArbitrary a, Arbitrary b) => Arbitrary (a -> b) where
-  arbitrary = promote (`coarbitrary` arbitrary)
+  arbitrary = arbitrary1
 
 instance Arbitrary () where
   arbitrary = return ()
@@ -341,27 +418,37 @@ instance Arbitrary Ordering where
   shrink LT = [EQ]
   shrink EQ = []
 
-instance Arbitrary a => Arbitrary (Maybe a) where
-  arbitrary = frequency [(1, return Nothing), (3, liftM Just arbitrary)]
+instance Arbitrary1 Maybe where
+  liftArbitrary arb = frequency [(1, return Nothing), (3, liftM Just arb)]
 
-  shrink (Just x) = Nothing : [ Just x' | x' <- shrink x ]
-  shrink _        = []
+  liftShrink shr (Just x) = Nothing : [ Just x' | x' <- shr x ]
+  liftShrink _   Nothing  = []
+
+instance Arbitrary a => Arbitrary (Maybe a) where
+  arbitrary = arbitrary1
+  shrink = shrink1
+
+instance Arbitrary2 Either where
+  liftArbitrary2 arbA arbB = oneof [liftM Left arbA, liftM Right arbB]
+
+  liftShrink2 shrA _ (Left x)  = [ Left  x' | x' <- shrA x ]
+  liftShrink2 _ shrB (Right y) = [ Right y' | y' <- shrB y ]
+
+instance Arbitrary a => Arbitrary1 (Either a) where
+  liftArbitrary = liftArbitrary2 arbitrary
+  liftShrink = liftShrink2 shrink
 
 instance (Arbitrary a, Arbitrary b) => Arbitrary (Either a b) where
-  arbitrary = oneof [liftM Left arbitrary, liftM Right arbitrary]
+  arbitrary = arbitrary2
+  shrink = shrink2
 
-  shrink (Left x)  = [ Left  x' | x' <- shrink x ]
-  shrink (Right y) = [ Right y' | y' <- shrink y ]
+instance Arbitrary1 [] where
+  liftArbitrary = listOf
+  liftShrink = shrinkList
 
 instance Arbitrary a => Arbitrary [a] where
-  arbitrary = listOf arbitrary
-  shrink xs = shrinkList shrink xs
-
-#ifndef NO_NONEMPTY
-instance Arbitrary a => Arbitrary (NonEmpty a) where
-  arbitrary = liftM2 (:|) arbitrary arbitrary
-  shrink (x :| xs) = mapMaybe nonEmpty . shrinkList shrink $ x : xs
-#endif
+  arbitrary = arbitrary1
+  shrink = shrink1
 
 -- | Shrink a list of values given a shrinking function for individual values.
 shrinkList :: (a -> [a]) -> [a] -> [[a]]
@@ -405,14 +492,19 @@ instance HasResolution a => Arbitrary (Fixed a) where
   shrink    = shrinkRealFrac
 #endif
 
-instance (Arbitrary a, Arbitrary b)
-      => Arbitrary (a,b)
- where
-  arbitrary = liftM2 (,) arbitrary arbitrary
+instance Arbitrary2 (,) where
+  liftArbitrary2 = liftM2 (,)
+  liftShrink2 shrA shrB (x, y) =
+       [ (x', y) | x' <- shrA x ]
+    ++ [ (x, y') | y' <- shrB y ]
 
-  shrink (x, y) =
-       [ (x', y) | x' <- shrink x ]
-    ++ [ (x, y') | y' <- shrink y ]
+instance (Arbitrary a) => Arbitrary1 ((,) a) where
+  liftArbitrary = liftArbitrary2 arbitrary
+  liftShrink = liftShrink2 shrink
+
+instance (Arbitrary a, Arbitrary b) => Arbitrary (a,b) where
+  arbitrary = arbitrary2
+  shrink = shrink2
 
 instance (Arbitrary a, Arbitrary b, Arbitrary c)
       => Arbitrary (a,b,c)
@@ -517,12 +609,6 @@ instance Arbitrary Integer where
   arbitrary = arbitrarySizedIntegral
   shrink    = shrinkIntegral
 
-#ifndef NO_NATURALS
-instance Arbitrary Natural where
-  arbitrary = arbitrarySizedNatural
-  shrink    = shrinkIntegral
-#endif
-
 instance Arbitrary Int where
   arbitrary = arbitrarySizedIntegral
   shrink    = shrinkIntegral
@@ -564,22 +650,26 @@ instance Arbitrary Word64 where
   shrink    = shrinkIntegral
 
 instance Arbitrary Char where
-  arbitrary = chr `fmap` oneof [choose (0,127), choose (0,255)]
-  shrink c  = filter (<. c) $ nub
+  arbitrary =
+    frequency
+      [(3, arbitraryASCIIChar),
+       (1, arbitraryUnicodeChar)]
+
+  shrink c = filter (<. c) $ nub
             $ ['a','b','c']
-           ++ [ toLower c | isUpper c ]
-           ++ ['A','B','C']
-           ++ ['1','2','3']
-           ++ [' ','\n']
-   where
-    a <. b  = stamp a < stamp b
-    stamp a = ( (not (isLower a)
-              , not (isUpper a)
-              , not (isDigit a))
-              , (not (a==' ')
-              , not (isSpace a)
-              , a)
-              )
+            ++ [ toLower c | isUpper c ]
+            ++ ['A','B','C']
+            ++ ['1','2','3']
+            ++ [' ','\n']
+     where
+      a <. b  = stamp a < stamp b
+      stamp a = ( (not (isLower a)
+                , not (isUpper a)
+                , not (isDigit a))
+                , (not (a==' ')
+                , not (isSpace a)
+                , a)
+                )
 
 instance Arbitrary Float where
   arbitrary = arbitrarySizedFractional
@@ -589,44 +679,230 @@ instance Arbitrary Double where
   arbitrary = arbitrarySizedFractional
   shrink    = shrinkRealFrac
 
+instance Arbitrary CChar where
+  arbitrary = arbitrarySizedBoundedIntegral
+  shrink = shrinkIntegral
+
+instance Arbitrary CSChar where
+  arbitrary = arbitrarySizedBoundedIntegral
+  shrink = shrinkIntegral
+
+instance Arbitrary CUChar where
+  arbitrary = arbitrarySizedBoundedIntegral
+  shrink = shrinkIntegral
+
+instance Arbitrary CShort where
+  arbitrary = arbitrarySizedBoundedIntegral
+  shrink = shrinkIntegral
+
+instance Arbitrary CUShort where
+  arbitrary = arbitrarySizedBoundedIntegral
+  shrink = shrinkIntegral
+
+instance Arbitrary CInt where
+  arbitrary = arbitrarySizedBoundedIntegral
+  shrink = shrinkIntegral
+
+instance Arbitrary CUInt where
+  arbitrary = arbitrarySizedBoundedIntegral
+  shrink = shrinkIntegral
+
+instance Arbitrary CLong where
+  arbitrary = arbitrarySizedBoundedIntegral
+  shrink = shrinkIntegral
+
+instance Arbitrary CULong where
+  arbitrary = arbitrarySizedBoundedIntegral
+  shrink = shrinkIntegral
+
+instance Arbitrary CPtrdiff where
+  arbitrary = arbitrarySizedBoundedIntegral
+  shrink = shrinkIntegral
+
+instance Arbitrary CSize where
+  arbitrary = arbitrarySizedBoundedIntegral
+  shrink = shrinkIntegral
+
+instance Arbitrary CWchar where
+  arbitrary = arbitrarySizedBoundedIntegral
+  shrink = shrinkIntegral
+
+instance Arbitrary CSigAtomic where
+  arbitrary = arbitrarySizedBoundedIntegral
+  shrink = shrinkIntegral
+
+instance Arbitrary CLLong where
+  arbitrary = arbitrarySizedBoundedIntegral
+  shrink = shrinkIntegral
+
+instance Arbitrary CULLong where
+  arbitrary = arbitrarySizedBoundedIntegral
+  shrink = shrinkIntegral
+
+instance Arbitrary CIntPtr where
+  arbitrary = arbitrarySizedBoundedIntegral
+  shrink = shrinkIntegral
+
+instance Arbitrary CUIntPtr where
+  arbitrary = arbitrarySizedBoundedIntegral
+  shrink = shrinkIntegral
+
+instance Arbitrary CIntMax where
+  arbitrary = arbitrarySizedBoundedIntegral
+  shrink = shrinkIntegral
+
+instance Arbitrary CUIntMax where
+  arbitrary = arbitrarySizedBoundedIntegral
+  shrink = shrinkIntegral
+
+#ifndef NO_NEWTYPE_DERIVING
+-- The following four types have no Bounded instance,
+-- so we fake it by discovering the bounds at runtime.
+instance Arbitrary CClock where
+  arbitrary = fmap unBounds arbitrary
+  shrink = shrinkMap unBounds Bounds
+
+instance Arbitrary CTime where
+  arbitrary = fmap unBounds arbitrary
+  shrink = shrinkMap unBounds Bounds
+
+#ifndef NO_FOREIGN_C_USECONDS
+instance Arbitrary CUSeconds where
+  arbitrary = fmap unBounds arbitrary
+  shrink = shrinkMap unBounds Bounds
+
+instance Arbitrary CSUSeconds where
+  arbitrary = fmap unBounds arbitrary
+  shrink = shrinkMap unBounds Bounds
+#endif
+
+newtype Bounds a = Bounds { unBounds :: a }
+  deriving (Eq, Ord, Num, Enum, Real, Show)
+
+instance (Ord a, Num a) => Bounded (Bounds a) where
+  -- assume max has all 1s in binary expansion
+  maxBound = maximum (nubIterate (\x -> 2*x+1) 1)
+  -- assume min has a leading 1 and rest 0s in binary expansion (or is 0)
+  minBound = minimum (0:nubIterate (*2) 1)
+
+instance (Num a, Real a, Enum a) => Integral (Bounds a) where
+  toInteger = fromIntegral . fromEnum
+  x `quotRem` y =
+    let (z, w) = toInteger x `quotRem` toInteger y in
+    (fromInteger z, fromInteger w)
+
+instance (Ord a, Num a, Real a, Enum a) => Arbitrary (Bounds a) where
+  arbitrary = arbitrarySizedBoundedIntegral
+  shrink = shrinkIntegral
+
+-- Like iterate, but stop when you reach an existing value.
+nubIterate :: Eq a => (a -> a) -> a -> [a]
+nubIterate f x = iter [] x
+  where
+    iter xs x
+      | x `elem` xs = []
+      | otherwise = x:iter (x:xs) (f x)
+#endif
+
+instance Arbitrary CFloat where
+  arbitrary = arbitrarySizedFractional
+  shrink = shrinkRealFrac
+
+instance Arbitrary CDouble where
+  arbitrary = arbitrarySizedFractional
+  shrink = shrinkRealFrac
+
 -- Arbitrary instances for container types
 instance (Ord a, Arbitrary a) => Arbitrary (Set.Set a) where
   arbitrary = fmap Set.fromList arbitrary
   shrink = map Set.fromList . shrink . Set.toList
+instance (Ord k, Arbitrary k) => Arbitrary1 (Map.Map k) where
+  liftArbitrary = fmap Map.fromList . liftArbitrary . liftArbitrary
+  liftShrink shr = map Map.fromList . liftShrink (liftShrink shr) . Map.toList
 instance (Ord k, Arbitrary k, Arbitrary v) => Arbitrary (Map.Map k v) where
-  arbitrary = fmap Map.fromList arbitrary
-  shrink = map Map.fromList . shrink . Map.toList
+  arbitrary = arbitrary1
+  shrink = shrink1
 instance Arbitrary IntSet.IntSet where
   arbitrary = fmap IntSet.fromList arbitrary
   shrink = map IntSet.fromList . shrink . IntSet.toList
+instance Arbitrary1 IntMap.IntMap where
+  liftArbitrary = fmap IntMap.fromList . liftArbitrary . liftArbitrary
+  liftShrink shr = map IntMap.fromList . liftShrink (liftShrink shr) . IntMap.toList
 instance Arbitrary a => Arbitrary (IntMap.IntMap a) where
-  arbitrary = fmap IntMap.fromList arbitrary
-  shrink = map IntMap.fromList . shrink . IntMap.toList
+  arbitrary = arbitrary1
+  shrink = shrink1
+instance Arbitrary1 Sequence.Seq where
+  liftArbitrary = fmap Sequence.fromList . liftArbitrary
+  liftShrink shr = map Sequence.fromList . liftShrink shr . toList
 instance Arbitrary a => Arbitrary (Sequence.Seq a) where
-  arbitrary = fmap Sequence.fromList arbitrary
-  shrink = map Sequence.fromList . shrink . toList
+  arbitrary = arbitrary1
+  shrink = shrink1
 
 -- Arbitrary instance for Ziplist
+instance Arbitrary1 ZipList where
+  liftArbitrary = fmap ZipList . liftArbitrary
+  liftShrink shr = map ZipList . liftShrink shr . getZipList
 instance Arbitrary a => Arbitrary (ZipList a) where
-  arbitrary = fmap ZipList arbitrary
-  shrink = map ZipList . shrink . getZipList
+  arbitrary = arbitrary1
+  shrink = shrink1
 
 #ifndef NO_TRANSFORMERS
 -- Arbitrary instance for transformers' Functors
+instance Arbitrary1 Identity where
+  liftArbitrary = fmap Identity
+  liftShrink shr = map Identity . shr . runIdentity
 instance Arbitrary a => Arbitrary (Identity a) where
-  arbitrary = fmap Identity arbitrary
-  shrink = map Identity . shrink . runIdentity
+  arbitrary = arbitrary1
+  shrink = shrink1
 
+instance Arbitrary2 Constant where
+  liftArbitrary2 arbA _ = fmap Constant arbA
+  liftShrink2 shrA _ = fmap Constant . shrA . getConstant
+instance Arbitrary a => Arbitrary1 (Constant a) where
+  liftArbitrary = liftArbitrary2 arbitrary
+  liftShrink = liftShrink2 shrink
+-- Have to be defined explicitly, as Constant is kind polymorphic
 instance Arbitrary a => Arbitrary (Constant a b) where
   arbitrary = fmap Constant arbitrary
   shrink = map Constant . shrink . getConstant
+
+instance (Arbitrary1 f, Arbitrary1 g) => Arbitrary1 (Product f g) where
+  liftArbitrary arb = liftM2 Pair (liftArbitrary arb) (liftArbitrary arb)
+  liftShrink shr (Pair f g) =
+    [ Pair f' g | f' <- liftShrink shr f ] ++
+    [ Pair f g' | g' <- liftShrink shr g ]
+instance (Arbitrary1 f, Arbitrary1 g, Arbitrary a) => Arbitrary (Product f g a) where
+  arbitrary = arbitrary1
+  shrink = shrink1
+
+instance (Arbitrary1 f, Arbitrary1 g) => Arbitrary1 (Compose f g) where
+  liftArbitrary = fmap Compose . liftArbitrary . liftArbitrary
+  liftShrink shr = map Compose . liftShrink (liftShrink shr) . getCompose
+instance (Arbitrary1 f, Arbitrary1 g, Arbitrary a) => Arbitrary (Compose f g a) where
+  arbitrary = arbitrary1
+  shrink = shrink1
 #endif
 
 -- Arbitrary instance for Const
+instance Arbitrary2 Const where
+  liftArbitrary2 arbA _ = fmap Const arbA
+  liftShrink2 shrA _ = fmap Const . shrA . getConst
+instance Arbitrary a => Arbitrary1 (Const a) where
+  liftArbitrary = liftArbitrary2 arbitrary
+  liftShrink = liftShrink2 shrink
+-- Have to be defined explicitly, as Const is kind polymorphic
 instance Arbitrary a => Arbitrary (Const a b) where
   arbitrary = fmap Const arbitrary
   shrink = map Const . shrink . getConst
 
+instance Arbitrary (m a) => Arbitrary (WrappedMonad m a) where
+  arbitrary = WrapMonad <$> arbitrary
+  shrink (WrapMonad a) = map WrapMonad (shrink a)
+
+instance Arbitrary (a b c) => Arbitrary (WrappedArrow a b c) where
+  arbitrary = WrapArrow <$> arbitrary
+  shrink (WrapArrow a) = map WrapArrow (shrink a)
+
 -- Arbitrary instances for Monoid
 instance Arbitrary a => Arbitrary (Monoid.Dual a) where
   arbitrary = fmap Monoid.Dual arbitrary
@@ -688,8 +964,35 @@ instance Arbitrary Version where
     , all (>=0) xs'
     ]
 
+instance Arbitrary QCGen where
+  arbitrary = MkGen (\g _ -> g)
+
+instance Arbitrary ExitCode where
+  arbitrary = frequency [(1, return ExitSuccess), (3, liftM ExitFailure arbitrary)]
+
+  shrink (ExitFailure x) = ExitSuccess : [ ExitFailure x' | x' <- shrink x ]
+  shrink _        = []
+
+
+
 -- ** Helper functions for implementing arbitrary
 
+-- | Apply a binary function to random arguments.
+applyArbitrary2 :: (Arbitrary a, Arbitrary b) => (a -> b -> r) -> Gen r
+applyArbitrary2 f = liftA2 f arbitrary arbitrary
+
+-- | Apply a ternary function to random arguments.
+applyArbitrary3
+  :: (Arbitrary a, Arbitrary b, Arbitrary c)
+  => (a -> b -> c -> r) -> Gen r
+applyArbitrary3 f = liftA3 f arbitrary arbitrary arbitrary
+
+-- | Apply a function of arity 4 to random arguments.
+applyArbitrary4
+  :: (Arbitrary a, Arbitrary b, Arbitrary c, Arbitrary d)
+  => (a -> b -> c -> d -> r) -> Gen r
+applyArbitrary4 f = applyArbitrary3 (uncurry f)
+
 -- | Generates an integral number. The number can be positive or negative
 -- and its maximum absolute value depends on the size parameter.
 arbitrarySizedIntegral :: Integral a => Gen a
@@ -759,12 +1062,46 @@ arbitrarySizedBoundedIntegral =
        n <- choose (toInteger mn `max` (-k), toInteger mx `min` k)
        return (fromInteger n)
 
+-- ** Generators for various kinds of character
+
+-- | Generates any Unicode character (but not a surrogate)
+arbitraryUnicodeChar :: Gen Char
+arbitraryUnicodeChar =
+  arbitraryBoundedEnum `suchThat` (not . isSurrogate)
+  where
+    isSurrogate c = generalCategory c == Surrogate
+
+-- | Generates a random ASCII character (0-127).
+arbitraryASCIIChar :: Gen Char
+arbitraryASCIIChar = choose ('\0', '\127')
+
+-- | Generates a printable Unicode character.
+arbitraryPrintableChar :: Gen Char
+arbitraryPrintableChar = arbitrary `suchThat` isPrint
+
 -- ** Helper functions for implementing shrink
 
 -- | Returns no shrinking alternatives.
 shrinkNothing :: a -> [a]
 shrinkNothing _ = []
 
+-- | Map a shrink function to another domain. This is handy if your data type
+-- has special invariants, but is /almost/ isomorphic to some other type.
+--
+-- @
+-- shrinkOrderedList :: (Ord a, Arbitrary a) => [a] -> [[a]]
+-- shrinkOrderedList = shrinkMap sort id
+--
+-- shrinkSet :: (Ord a, Arbitrary a) => Set a -> Set [a]
+-- shrinkSet = shrinkMap fromList toList
+-- @
+shrinkMap :: Arbitrary a => (a -> b) -> (b -> a) -> b -> [b]
+shrinkMap f g = shrinkMapBy f g shrink
+
+-- | Non-overloaded version of `shrinkMap`.
+shrinkMapBy :: (a -> b) -> (b -> a) -> (a -> [a]) -> b -> [b]
+shrinkMapBy f g shr = map f . shr . g
+
 -- | Shrink an integral number.
 shrinkIntegral :: Integral a => a -> [a]
 shrinkIntegral x =
@@ -892,11 +1229,6 @@ instance CoArbitrary a => CoArbitrary [a] where
   coarbitrary []     = variant 0
   coarbitrary (x:xs) = variant 1 . coarbitrary (x,xs)
 
-#ifndef NO_NONEMPTY
-instance CoArbitrary a => CoArbitrary (NonEmpty a) where
-  coarbitrary (x :| xs) = coarbitrary (x, xs)
-#endif
-
 instance (Integral a, CoArbitrary a) => CoArbitrary (Ratio a) where
   coarbitrary r = coarbitrary (numerator r,denominator r)
 
@@ -943,11 +1275,6 @@ instance (CoArbitrary a, CoArbitrary b, CoArbitrary c, CoArbitrary d, CoArbitrar
 instance CoArbitrary Integer where
   coarbitrary = coarbitraryIntegral
 
-#ifndef NO_NATURALS
-instance CoArbitrary Natural where
-  coarbitrary = coarbitraryIntegral
-#endif
-
 instance CoArbitrary Int where
   coarbitrary = coarbitraryIntegral
 
@@ -1084,7 +1411,7 @@ vector k = vectorOf k arbitrary
 orderedList :: (Ord a, Arbitrary a) => Gen [a]
 orderedList = sort `fmap` arbitrary
 
--- | Generate an infinite list.
+-- | Generates an infinite list.
 infiniteList :: Arbitrary a => Gen [a]
 infiniteList = infiniteListOf arbitrary
 
