@@ -1,4 +1,11 @@
 {-# LANGUAGE CPP #-}
+#ifndef NO_SAFE_HASKELL
+#if !defined(NO_ST_MONAD) && !(MIN_VERSION_base(4,8,0))
+{-# LANGUAGE Trustworthy #-}
+#else
+{-# LANGUAGE Safe #-}
+#endif
+#endif
 #ifndef NO_ST_MONAD
 {-# LANGUAGE Rank2Types #-}
 #endif
@@ -83,6 +90,10 @@ import Control.Monad.IO.Class
 import Control.Monad.Trans.Class
 #endif
 
+#ifndef NO_MONADFAIL
+import qualified Control.Monad.Fail as Fail
+#endif
+
 --------------------------------------------------------------------------
 -- type PropertyM
 
@@ -95,17 +106,29 @@ import Control.Monad.Trans.Class
 newtype PropertyM m a =
   MkPropertyM { unPropertyM :: (a -> Gen (m Property)) -> Gen (m Property) }
 
+bind :: PropertyM m a -> (a -> PropertyM m b) -> PropertyM m b
+MkPropertyM m `bind` f = MkPropertyM (\k -> m (\a -> unPropertyM (f a) k))
+
+fail_ :: Monad m => String -> PropertyM m a
+fail_ s = stop (failed { reason = s })
+
 instance Functor (PropertyM m) where
   fmap f (MkPropertyM m) = MkPropertyM (\k -> m (k . f))
 
-instance Monad m => Applicative (PropertyM m) where
-  pure = return
-  (<*>) = liftM2 ($)
+instance Applicative (PropertyM m) where
+  pure x = MkPropertyM (\k -> k x)
+  mf <*> mx =
+    mf `bind` \f -> mx `bind` \x -> pure (f x)
 
 instance Monad m => Monad (PropertyM m) where
-  return x            = MkPropertyM (\k -> k x)
-  MkPropertyM m >>= f = MkPropertyM (\k -> m (\a -> unPropertyM (f a) k))
-  fail s              = stop (failed { reason = s })
+  return = pure
+  (>>=) = bind
+  fail = fail_
+
+#ifndef NO_MONADFAIL
+instance Monad m => Fail.MonadFail (PropertyM m) where
+  fail = fail_
+#endif
 
 #ifndef NO_TRANSFORMERS
 instance MonadTrans PropertyM where
@@ -201,11 +224,11 @@ monitor f = MkPropertyM (\k -> (f `liftM`) `fmap` (k ()))
 
 -- run functions
 
-monadic :: Monad m => (m Property -> Property) -> PropertyM m a -> Property
+monadic :: (Testable a, Monad m) => (m Property -> Property) -> PropertyM m a -> Property
 monadic runner m = property (fmap runner (monadic' m))
 
-monadic' :: Monad m => PropertyM m a -> Gen (m Property)
-monadic' (MkPropertyM m) = m (const (return (return (property True))))
+monadic' :: (Testable a, Monad m) => PropertyM m a -> Gen (m Property)
+monadic' (MkPropertyM m) = m (\prop -> return (return (property prop)))
 
 -- | Runs the property monad for 'IO'-computations.
 --
@@ -221,7 +244,7 @@ monadic' (MkPropertyM m) = m (const (return (return (property True))))
 -- >>> quickCheck prop_cat
 -- +++ OK, passed 100 tests.
 --
-monadicIO :: PropertyM IO a -> Property
+monadicIO :: Testable a => PropertyM IO a -> Property
 monadicIO = monadic ioProperty
 
 #ifndef NO_ST_MONAD
@@ -240,7 +263,7 @@ monadicIO = monadic ioProperty
 -- >>> quickCheck prop_sortST
 -- +++ OK, passed 100 tests.
 --
-monadicST :: (forall s. PropertyM (ST s) a) -> Property
+monadicST :: Testable a => (forall s. PropertyM (ST s) a) -> Property
 monadicST m = property (runSTGen (monadic' m))
 
 runSTGen :: (forall s. Gen (ST s a)) -> Gen a
