@@ -1,5 +1,8 @@
 -- | Combinators for constructing properties.
 {-# LANGUAGE CPP #-}
+#ifndef NO_TYPEABLE
+{-# LANGUAGE DeriveDataTypeable #-}
+#endif
 #ifndef NO_SAFE_HASKELL
 {-# LANGUAGE Safe #-}
 #endif
@@ -11,7 +14,7 @@ module Test.QuickCheck.Property where
 import Test.QuickCheck.Gen
 import Test.QuickCheck.Gen.Unsafe
 import Test.QuickCheck.Arbitrary
-import Test.QuickCheck.Text( showErr, isOneLine, putLine )
+import Test.QuickCheck.Text( isOneLine, putLine )
 import Test.QuickCheck.Exception
 import Test.QuickCheck.State hiding (labels)
 
@@ -25,6 +28,12 @@ import qualified Data.Map as Map
 import Data.Map(Map)
 import qualified Data.Set as Set
 import Data.Set(Set)
+#ifndef NO_DEEPSEQ
+import Control.DeepSeq
+#endif
+#ifndef NO_TYPEABLE
+import Data.Typeable (Typeable)
+#endif
 
 --------------------------------------------------------------------------
 -- fixities
@@ -68,8 +77,17 @@ infixr 1 .||.
 
 -- | The type of properties.
 newtype Property = MkProperty { unProperty :: Gen Prop }
+#ifndef NO_TYPEABLE
+  deriving (Typeable)
+#endif
 
--- | The class of things which can be tested, i.e. turned into a property.
+-- | The class of properties, i.e., types which QuickCheck knows how to test.
+-- Typically a property will be a function returning 'Bool' or 'Property'.
+--
+-- If a property does no quantification, i.e. has no
+-- parameters and doesn't use 'forAll', it will only be tested once.
+-- This may not be what you want if your property is an @IO Bool@.
+-- You can change this behaviour using the 'again' combinator.
 class Testable prop where
   -- | Convert the thing to a property.
   property :: prop -> Property
@@ -81,6 +99,14 @@ data Discard = Discard
 instance Testable Discard where
   property _ = property rejected
 
+-- This instance is here to make it easier to turn IO () into a Property.
+instance Testable () where
+  property = property . liftUnit
+    where
+      -- N.B. the unit gets forced only inside 'property',
+      -- so that we turn exceptions into test failures
+      liftUnit () = succeeded
+
 instance Testable Bool where
   property = property . liftBool
 
@@ -94,26 +120,22 @@ instance Testable prop => Testable (Gen prop) where
   property mp = MkProperty $ do p <- mp; unProperty (again p)
 
 instance Testable Property where
-  property = property . unProperty
+  property (MkProperty mp) = MkProperty $ do p <- mp; unProperty (property p)
 
--- | Do I/O inside a property. This can obviously lead to unrepeatable
--- testcases, so use with care.
-{-# DEPRECATED morallyDubiousIOProperty "Use ioProperty instead" #-}
+-- | Do I/O inside a property.
+{-# DEPRECATED morallyDubiousIOProperty "Use 'ioProperty' instead" #-}
 morallyDubiousIOProperty :: Testable prop => IO prop -> Property
-morallyDubiousIOProperty = ioProperty -- Silly names aren't all they're cracked up to be :)
+morallyDubiousIOProperty = ioProperty
 
--- | Do I/O inside a property. This can obviously lead to unrepeatable
--- testcases, so use with care.
+-- | Do I/O inside a property.
 --
--- For more advanced monadic testing you may want to look at
--- "Test.QuickCheck.Monadic".
---
--- Note that if you use 'ioProperty' on a property of type @IO Bool@,
--- or more generally a property that does no quantification, the property
--- will only be executed once. To test the property repeatedly you must
--- use the 'again' combinator.
+-- Warning: any random values generated inside of the argument to @ioProperty@
+-- will not currently be shrunk. For best results, generate all random values
+-- before calling @ioProperty@.
 ioProperty :: Testable prop => IO prop -> Property
-ioProperty = MkProperty . fmap (MkProp . ioRose . fmap unProp) . promote . fmap (unProperty . property)
+ioProperty =
+  MkProperty . fmap (MkProp . ioRose . fmap unProp) .
+  promote . fmap (unProperty . noShrinking)
 
 instance (Arbitrary a, Show a, Testable prop) => Testable (a -> prop) where
   property f = forAllShrink arbitrary shrink f
@@ -194,14 +216,16 @@ data CallbackKind = Counterexample    -- ^ Affected by the 'verbose' combinator
 -- | The result of a single test.
 data Result
   = MkResult
-  { ok           :: Maybe Bool        -- ^ result of the test case; Nothing = discard
-  , expect       :: Bool              -- ^ indicates what the expected result of the property is
-  , reason       :: String            -- ^ a message indicating what went wrong
-  , theException :: Maybe AnException -- ^ the exception thrown, if any
-  , abort        :: Bool              -- ^ if True, the test should not be repeated
-  , labels       :: Map String Int    -- ^ all labels used by this property
-  , stamp        :: Set String        -- ^ the collected values for this test case
-  , callbacks    :: [Callback]        -- ^ the callbacks for this test case
+  { ok            :: Maybe Bool        -- ^ result of the test case; Nothing = discard
+  , expect        :: Bool              -- ^ indicates what the expected result of the property is
+  , reason        :: String            -- ^ a message indicating what went wrong
+  , theException  :: Maybe AnException -- ^ the exception thrown, if any
+  , abort         :: Bool              -- ^ if True, the test should not be repeated
+  , maybeNumTests :: Maybe Int         -- ^ stop after this many tests
+  , labels        :: Map String Int    -- ^ all labels used by this property
+  , stamp         :: Set String        -- ^ the collected labels for this test case
+  , callbacks     :: [Callback]        -- ^ the callbacks for this test case
+  , testCase      :: [String]          -- ^ the generated test case
   }
 
 exception :: String -> AnException -> Result
@@ -226,14 +250,16 @@ succeeded, failed, rejected :: Result
   where
     result =
       MkResult
-      { ok           = undefined
-      , expect       = True
-      , reason       = ""
-      , theException = Nothing
-      , abort        = True
-      , labels       = Map.empty
-      , stamp        = Set.empty
-      , callbacks    = []
+      { ok            = undefined
+      , expect        = True
+      , reason        = ""
+      , theException  = Nothing
+      , abort         = True
+      , maybeNumTests = Nothing
+      , labels        = Map.empty
+      , stamp         = Set.empty
+      , callbacks     = []
+      , testCase      = []
       }
 
 --------------------------------------------------------------------------
@@ -263,8 +289,8 @@ mapProp f = MkProperty . fmap f . unProperty . property
 mapSize :: Testable prop => (Int -> Int) -> prop -> Property
 mapSize f p = MkProperty (sized ((`resize` unProperty (property p)) . f))
 
--- | Shrinks the argument to property if it fails. Shrinking is done
--- automatically for most types. This is only needed when you want to
+-- | Shrinks the argument to a property if it fails. Shrinking is done
+-- automatically for most types. This function is only needed when you want to
 -- override the default behavior.
 shrinking :: Testable prop =>
              (a -> [a])  -- ^ 'shrink'-like function.
@@ -283,18 +309,27 @@ noShrinking = mapRoseResult (onRose (\res _ -> MkRose res []))
 callback :: Testable prop => Callback -> prop -> Property
 callback cb = mapTotalResult (\res -> res{ callbacks = cb : callbacks res })
 
--- | Adds the given string to the counterexample.
+-- | Adds the given string to the counterexample if the property fails.
 counterexample :: Testable prop => String -> prop -> Property
 counterexample s =
-  callback $ PostFinalFailure Counterexample $ \st _res -> do
-    res <- tryEvaluateIO (putLine (terminal st) s)
+  mapTotalResult (\res -> res{ testCase = s:testCase res }) .
+  callback (PostFinalFailure Counterexample $ \st _res -> do
+    s <- showCounterexample s
+    putLine (terminal st) s)
+
+showCounterexample :: String -> IO String
+showCounterexample s = do
+  let force [] = return ()
+      force (x:xs) = x `seq` force xs
+  res <- tryEvaluateIO (force s)
+  return $
     case res of
       Left err ->
-        putLine (terminal st) (formatException "Exception thrown while printing test case" err)
+        formatException "Exception thrown while showing test case" err
       Right () ->
-        return ()
+        s
 
--- | Adds the given string to the counterexample.
+-- | Adds the given string to the counterexample if the property fails.
 {-# DEPRECATED printTestCase "Use counterexample instead" #-}
 printTestCase :: Testable prop => String -> prop -> Property
 printTestCase = counterexample
@@ -321,7 +356,8 @@ verbose :: Testable prop => prop -> Property
 verbose = mapResult (\res -> res { callbacks = newCallbacks (callbacks res) ++ callbacks res })
   where newCallbacks cbs =
           PostTest Counterexample (\st res -> putLine (terminal st) (status res ++ ":")):
-          [ PostTest Counterexample f | PostFinalFailure Counterexample f <- cbs ]
+          [ PostTest Counterexample f | PostFinalFailure Counterexample f <- cbs ] ++
+          [ PostTest Counterexample (\st res -> putLine (terminal st) "") ]
         status MkResult{ok = Just True} = "Passed"
         status MkResult{ok = Just False} = "Failed"
         status MkResult{ok = Nothing} = "Skipped (precondition false)"
@@ -332,25 +368,79 @@ expectFailure :: Testable prop => prop -> Property
 expectFailure = mapTotalResult (\res -> res{ expect = False })
 
 -- | Modifies a property so that it only will be tested once.
+-- Opposite of 'again'.
 once :: Testable prop => prop -> Property
 once = mapTotalResult (\res -> res{ abort = True })
 
--- | Undoes the effect of 'once'.
+-- | Modifies a property so that it will be tested repeatedly.
+-- Opposite of 'once'.
 again :: Testable prop => prop -> Property
 again = mapTotalResult (\res -> res{ abort = False })
 
+-- | Configures how many times a property will be tested.
+--
+-- For example,
+--
+-- > quickCheck (withMaxSuccess 1000 p)
+--
+-- will test @p@ up to 1000 times.
+withMaxSuccess :: Testable prop => Int -> prop -> Property
+withMaxSuccess n = n `seq` mapTotalResult (\res -> res{ maybeNumTests = Just n })
+
 -- | Attaches a label to a property. This is used for reporting
 -- test case distribution.
+--
+-- For example:
+--
+-- > prop_reverse_reverse :: [Int] -> Property
+-- > prop_reverse_reverse xs =
+-- >   label ("length of input is " ++ show (length xs)) $
+-- >     reverse (reverse xs) === xs
+--
+-- >>> quickCheck prop_reverse_reverse
+-- +++ OK, passed 100 tests:
+-- 7% length of input is 7
+-- 6% length of input is 3
+-- 5% length of input is 4
+-- 4% length of input is 6
+-- ...
 label :: Testable prop => String -> prop -> Property
 label s = classify True s
 
--- | Labels a property with a value:
+-- | Attaches a label to a property. This is used for reporting
+-- test case distribution.
 --
 -- > collect x = label (show x)
+--
+-- For example:
+--
+-- > prop_reverse_reverse :: [Int] -> Property
+-- > prop_reverse_reverse xs =
+-- >   collect (length xs) $
+-- >     reverse (reverse xs) === xs
+--
+-- >>> quickCheck prop_reverse_reverse
+-- +++ OK, passed 100 tests:
+-- 7% 7
+-- 6% 3
+-- 5% 4
+-- 4% 6
+-- ...
 collect :: (Show a, Testable prop) => a -> prop -> Property
 collect x = label (show x)
 
--- | Conditionally labels test case.
+-- | Records how many test cases satisfy a given condition.
+--
+-- For example:
+--
+-- > prop_sorted_sort :: [Int] -> Property
+-- > prop_sorted_sort xs =
+-- >   sorted xs ==>
+-- >   classify (length xs > 1) "non-trivial" $
+-- >   sort xs === xs
+--
+-- >>> quickCheck prop_sorted_sort
+-- +++ OK, passed 100 tests (22% non-trivial).
 classify :: Testable prop =>
             Bool    -- ^ @True@ if the test case should be labelled.
          -> String  -- ^ Label.
@@ -360,6 +450,17 @@ classify b s = cover b 0 s
 -- | Checks that at least the given proportion of /successful/ test
 -- cases belong to the given class. Discarded tests (i.e. ones
 -- with a false precondition) do not affect coverage.
+--
+-- For example:
+--
+-- > prop_sorted_sort :: [Int] -> Property
+-- > prop_sorted_sort xs =
+-- >   sorted xs ==>
+-- >   cover (length xs > 1) 50 "non-trivial" $
+-- >   sort xs === xs
+--
+-- >>> quickCheck prop_sorted_sort
+-- *** Insufficient coverage after 100 tests (only 24% non-trivial, not 50%).
 cover :: Testable prop =>
          Bool   -- ^ @True@ if the test case belongs to the class.
       -> Int    -- ^ The required percentage (0-100) of test cases.
@@ -385,16 +486,16 @@ True  ==> p = property p
 -- the given number of microseconds.
 within :: Testable prop => Int -> prop -> Property
 within n = mapRoseResult f
-  -- We rely on the fact that the property will catch the timeout
-  -- exception and turn it into a failed test case.
   where
     f rose = ioRose $ do
-      let m `orError` x = fmap (fromMaybe (error x)) m
+      let m `orError` x = fmap (fromMaybe x) m
       MkRose res roses <- timeout n (reduceRose rose) `orError`
-                          "within: timeout exception not caught in Rose Result"
+        return timeoutResult
       res' <- timeout n (protectResult (return res)) `orError`
-              "within: timeout exception not caught in Result"
+        timeoutResult
       return (MkRose res' (map f roses))
+
+    timeoutResult = failed { reason = "Timeout" }
 #ifdef NO_TIMEOUT
     timeout _ = fmap Just
 #endif
@@ -498,12 +599,16 @@ disjoin ps =
                    -- The following three fields are not important because the
                    -- test case has failed anyway
                    abort = False,
+                   maybeNumTests = Nothing,
                    labels = Map.empty,
                    stamp = Set.empty,
                    callbacks =
                      callbacks result1 ++
                      [PostFinalFailure Counterexample $ \st _res -> putLine (terminal st) ""] ++
-                     callbacks result2 }
+                     callbacks result2,
+                   testCase =
+                     testCase result1 ++
+                     testCase result2 }
                Nothing -> result2
          Nothing -> do
            result2 <- q
@@ -521,8 +626,17 @@ disjoin ps =
 infix 4 ===
 (===) :: (Eq a, Show a) => a -> a -> Property
 x === y =
-  counterexample (show x ++ " /= " ++ show y) (x == y)
+  counterexample (show x ++ interpret res ++ show y) res
+  where
+    res = x == y
+    interpret True  = " == "
+    interpret False = " /= "
 
+#ifndef NO_DEEPSEQ
+-- | Checks that a value is total, i.e., doesn't crash when evaluated.
+total :: NFData a => a -> Property
+total x = property (rnf x)
+#endif
 
 --------------------------------------------------------------------------
 -- the end.
