@@ -3,7 +3,6 @@ module Test.QuickCheck.State where
 
 import Test.QuickCheck.Text
 import Test.QuickCheck.Random
-import qualified Data.Map as Map
 import Data.Map(Map)
 import Data.Set(Set)
 
@@ -17,9 +16,10 @@ data State
   -- static
   { terminal                  :: Terminal          -- ^ the current terminal
   , maxSuccessTests           :: Int               -- ^ maximum number of successful tests needed
-  , maxDiscardedTests         :: Int               -- ^ maximum number of tests that can be discarded
+  , maxDiscardedRatio         :: Int               -- ^ maximum number of discarded tests per successful test
   , computeSize               :: Int -> Int -> Int -- ^ how to compute the size of test cases from
                                                    --   #tests and #discarded tests
+  , numTotMaxShrinks          :: !Int              -- ^ How many shrinks to try before giving up
 
   -- dynamic
   , numSuccessTests           :: !Int              -- ^ the current number of tests that have succeeded
