@@ -34,10 +34,15 @@ import Data.Char
 
 import Data.List
   ( sort
+  , sortBy
   , group
-  , groupBy
   , intersperse
   )
+
+import Data.Maybe(fromMaybe)
+import Data.Ord(comparing)
+import Text.Printf(printf)
+
 --------------------------------------------------------------------------
 -- quickCheck
 
@@ -53,13 +58,18 @@ data Args
     -- If you want to store a test case permanently you should save
     -- the test case itself.
   , maxSuccess      :: Int
-    -- ^ Maximum number of successful tests before succeeding
+    -- ^ Maximum number of successful tests before succeeding. Testing stops
+    -- at the first failure. If all tests are passing and you want to run more tests,
+    -- increase this number.
   , maxDiscardRatio :: Int
     -- ^ Maximum number of discarded tests per successful test before giving up
   , maxSize         :: Int
     -- ^ Size to use for the biggest test cases
   , chatty          :: Bool
     -- ^ Whether to print anything
+  , maxShrinks      :: Int
+    -- ^ Maximum number of shrinks to before giving up. Setting this to zero
+    --   turns shrinking off.
   }
  deriving ( Show, Read )
 
@@ -68,38 +78,39 @@ data Result
   -- | A successful test run
   = Success
     { numTests       :: Int               -- ^ Number of tests performed
-    , labels         :: [(String,Int)]    -- ^ Labels and frequencies found during all successful tests
+    , labels         :: [(String,Double)] -- ^ Labels and frequencies found during all successful tests
     , output         :: String            -- ^ Printed output
     }
   -- | Given up
   | GaveUp
     { numTests       :: Int               --   Number of tests performed
-    , labels         :: [(String,Int)]    --   Labels and frequencies found during all successful tests
+    , labels         :: [(String,Double)] --   Labels and frequencies found during all successful tests
     , output         :: String            --   Printed output
     }
   -- | A failed test run
   | Failure
-    { numTests       :: Int               --   Number of tests performed
-    , numShrinks     :: Int               -- ^ Number of successful shrinking steps performed
-    , numShrinkTries :: Int               -- ^ Number of unsuccessful shrinking steps performed
-    , numShrinkFinal :: Int               -- ^ Number of unsuccessful shrinking steps performed since last successful shrink
-    , usedSeed       :: QCGen             -- ^ What seed was used
-    , usedSize       :: Int               -- ^ What was the test size
-    , reason         :: String            -- ^ Why did the property fail
-    , theException   :: Maybe AnException -- ^ The exception the property threw, if any
-    , labels         :: [(String,Int)]    --   Labels and frequencies found during all successful tests
-    , output         :: String            --   Printed output
+    { numTests        :: Int               --   Number of tests performed
+    , numShrinks      :: Int               -- ^ Number of successful shrinking steps performed
+    , numShrinkTries  :: Int               -- ^ Number of unsuccessful shrinking steps performed
+    , numShrinkFinal  :: Int               -- ^ Number of unsuccessful shrinking steps performed since last successful shrink
+    , usedSeed        :: QCGen             -- ^ What seed was used
+    , usedSize        :: Int               -- ^ What was the test size
+    , reason          :: String            -- ^ Why did the property fail
+    , theException    :: Maybe AnException -- ^ The exception the property threw, if any
+    , labels          :: [(String,Double)] --   Labels and frequencies found during all successful tests
+    , output          :: String            --   Printed output
+    , failingTestCase :: [String]          -- ^ The test case which provoked the failure
     }
   -- | A property that should have failed did not
   | NoExpectedFailure
     { numTests       :: Int               --   Number of tests performed
-    , labels         :: [(String,Int)]    --   Labels and frequencies found during all successful tests
+    , labels         :: [(String,Double)] --   Labels and frequencies found during all successful tests
     , output         :: String            --   Printed output
     }
  -- | The tests passed but a use of 'cover' had insufficient coverage
  | InsufficientCoverage
     { numTests       :: Int               --   Number of tests performed
-    , labels         :: [(String,Int)]    --   Labels and frequencies found during all successful tests
+    , labels         :: [(String,Double)] --   Labels and frequencies found during all successful tests
     , output         :: String            --   Printed output
     }
  deriving ( Show )
@@ -117,10 +128,13 @@ stdArgs = Args
   , maxDiscardRatio = 10
   , maxSize         = 100
   , chatty          = True
--- noShrinking flag?
+  , maxShrinks      = maxBound
   }
 
 -- | Tests a property and prints the results to 'stdout'.
+--
+-- By default up to 100 tests are performed, which may not be enough
+-- to find all bugs. To run more tests, use 'withMaxSuccess'.
 quickCheck :: Testable prop => prop -> IO ()
 quickCheck p = quickCheckWith stdArgs p
 
@@ -140,10 +154,11 @@ quickCheckWithResult a p = (if chatty a then withStdioTerminal else withNullTerm
               Just (rnd,_) -> return rnd
      test MkState{ terminal                  = tm
                  , maxSuccessTests           = maxSuccess a
-                 , maxDiscardedTests         = maxDiscardRatio a * maxSuccess a
+                 , maxDiscardedRatio         = maxDiscardRatio a
                  , computeSize               = case replay a of
                                                  Nothing    -> computeSize'
                                                  Just (_,s) -> computeSize' `at0` s
+                 , numTotMaxShrinks          = maxShrinks a
                  , numSuccessTests           = 0
                  , numDiscardedTests         = 0
                  , numRecentlyDiscardedTests = 0
@@ -192,9 +207,12 @@ verboseCheckWithResult a p = quickCheckWithResult a (verbose p)
 
 test :: State -> (QCGen -> Int -> Prop) -> IO Result
 test st f
-  | numSuccessTests st   >= maxSuccessTests st   = doneTesting st f
-  | numDiscardedTests st >= maxDiscardedTests st = giveUp st f
-  | otherwise                                    = runATest st f
+  | numSuccessTests st   >= maxSuccessTests st =
+    doneTesting st f
+  | numDiscardedTests st >= maxDiscardedRatio st * maxSuccessTests st =
+    giveUp st f
+  | otherwise =
+    runATest st f
 
 doneTesting :: State -> (QCGen -> Int -> Prop) -> IO Result
 doneTesting st _f
@@ -206,7 +224,7 @@ doneTesting st _f
        ++ " tests (expected failure)"
         )
       finished NoExpectedFailure
-  | insufficientCoverage st = do
+  | not (null (insufficientlyCovered st)) = do
       putPart (terminal st)
         ( bold ("*** Insufficient coverage after ")
        ++ show (numSuccessTests st)
@@ -264,59 +282,102 @@ runATest st f =
            | otherwise = x:xs
 
      case res of
-       MkResult{ok = Just True, stamp = stamp, expect = expect} -> -- successful test
+       MkResult{ok = Just True, stamp = stamp, expect = expect, maybeNumTests = mnt} -> -- successful test
          do continue doneTesting
               st{ numSuccessTests           = numSuccessTests st + 1
                 , numRecentlyDiscardedTests = 0
+                , maxSuccessTests           = fromMaybe (maxSuccessTests st) mnt
                 , randomSeed                = rnd2
                 , S.labels                  = Map.unionWith max (S.labels st) (P.labels res)
                 , collected                 = stamp `cons` collected st
                 , expectedFailure           = expect
                 } f
 
-       MkResult{ok = Nothing, expect = expect} -> -- discarded test
+       MkResult{ok = Nothing, expect = expect, maybeNumTests = mnt} -> -- discarded test
          do continue giveUp
               st{ numDiscardedTests         = numDiscardedTests st + 1
                 , numRecentlyDiscardedTests = numRecentlyDiscardedTests st + 1
+                , maxSuccessTests           = fromMaybe (maxSuccessTests st) mnt
                 , randomSeed                = rnd2
                 , S.labels                  = Map.unionWith max (S.labels st) (P.labels res)
                 , expectedFailure           = expect
                 } f
 
        MkResult{ok = Just False} -> -- failed test
-         do if expect res
-              then putPart (terminal st) (bold "*** Failed! ")
-              else putPart (terminal st) "+++ OK, failed as expected. "
-            (numShrinks, totFailed, lastFailed) <- foundFailure st res ts
+         do (numShrinks, totFailed, lastFailed, res) <- foundFailure st res ts
             theOutput <- terminalOutput (terminal st)
             if not (expect res) then
               return Success{ labels = summary st,
                               numTests = numSuccessTests st+1,
                               output = theOutput }
-             else
-              return Failure{ usedSeed       = randomSeed st -- correct! (this will be split first)
-                            , usedSize       = size
-                            , numTests       = numSuccessTests st+1
-                            , numShrinks     = numShrinks
-                            , numShrinkTries = totFailed
-                            , numShrinkFinal = lastFailed
-                            , output         = theOutput
-                            , reason         = P.reason res
-                            , theException   = P.theException res
-                            , labels         = summary st
+             else do
+              testCase <- mapM showCounterexample (P.testCase res)
+              return Failure{ usedSeed        = randomSeed st -- correct! (this will be split first)
+                            , usedSize        = size
+                            , numTests        = numSuccessTests st+1
+                            , numShrinks      = numShrinks
+                            , numShrinkTries  = totFailed
+                            , numShrinkFinal  = lastFailed
+                            , output          = theOutput
+                            , reason          = P.reason res
+                            , theException    = P.theException res
+                            , labels          = summary st
+                            , failingTestCase = testCase
                             }
  where
   (rnd1,rnd2) = split (randomSeed st)
 
-summary :: State -> [(String,Int)]
+failureSummary :: State -> P.Result -> String
+failureSummary st res = fst (failureSummaryAndReason st res)
+
+failureReason :: State -> P.Result -> [String]
+failureReason st res = snd (failureSummaryAndReason st res)
+
+failureSummaryAndReason :: State -> P.Result -> (String, [String])
+failureSummaryAndReason st res = (summary, full)
+  where
+    summary =
+      header ++
+      short 26 (oneLine reason ++ " ") ++
+      count True ++ "..."
+
+    full =
+      (header ++
+       (if isOneLine reason then reason ++ " " else "") ++
+       count False ++ ":"):
+      if isOneLine reason then [] else lines reason
+
+    reason = P.reason res
+
+    header =
+      if expect res then
+        bold "*** Failed! "
+      else "+++ OK, failed as expected. "
+
+    count full =
+      "(after " ++ number (numSuccessTests st+1) "test" ++
+      concat [
+        " and " ++
+        show (numSuccessShrinks st) ++
+        concat [ "." ++ show (numTryShrinks st) | showNumTryShrinks ] ++
+        " shrink" ++
+        (if numSuccessShrinks st == 1 && not showNumTryShrinks then "" else "s")
+        | numSuccessShrinks st > 0 || showNumTryShrinks ] ++
+      ")"
+      where
+        showNumTryShrinks = full && numTryShrinks st > 0
+
+summary :: State -> [(String, Double)]
 summary st = reverse
-           . sort
-           . map (\ss -> (head ss, (length ss * 100) `div` numSuccessTests st))
+           . sortBy (comparing snd)
+           . map (\ss -> (head ss, fromIntegral (length ss) * 100 / fromIntegral (numSuccessTests st)))
            . group
            . sort
-           $ [ concat (intersperse ", " (Set.toList s))
+           $ [ concat (intersperse ", " s')
              | s <- collected st
-             , not (Set.null s)
+               -- HACK: don't print out labels that were created by 'cover'.
+             , let s' = [ t | t <- Set.toList s, Map.lookup t (S.labels st) == Just 0 ]
+             , not (null s')
              ]
 
 success :: State -> IO ()
@@ -329,64 +390,62 @@ success st =
                  ++ ")."
                   )
     cases -> do putLine (terminal st) ":"
-                sequence_ [ putLine (terminal st) pt | pt <- cases ]
+                mapM_ (putLine $ terminal st) cases
  where
-  allLabels = reverse
-            . sort
-            . map (\ss -> (showP ((length ss * 100) `div` numSuccessTests st) ++ head ss))
-            . group
-            . sort
-            $ [ concat (intersperse ", " s')
-              | s <- collected st
-              , let s' = [ t | t <- Set.toList s, Map.lookup t (S.labels st) == Just 0 ]
-              , not (null s')
-              ]
+  allLabels :: [String]
+  allLabels = map (formatLabel (numSuccessTests st) True) (summary st)
 
-  covers = [ ("only " ++ show (labelPercentage l st) ++ "% " ++ l ++ ", not " ++ show reqP ++ "%")
-           | (l, reqP) <- Map.toList (S.labels st)
-           , labelPercentage l st < reqP
-           ]
+  covers :: [String]
+  covers = [ ("only " ++ formatLabel (numSuccessTests st) False (l, p) ++ ", not " ++ show reqP ++ "%")
+           | (l, reqP, p) <- insufficientlyCovered st ]
 
-  showP p = (if p < 10 then " " else "") ++ show p ++ "% "
+formatLabel :: Int -> Bool -> (String, Double) -> String
+formatLabel n pad (x, p) = showP pad p ++ " " ++ x
+ where
+  showP :: Bool -> Double -> String
+  showP pad p =
+    (if pad && p < 10 then " " else "") ++
+    printf "%.*f" places p ++ "%"
 
-labelPercentage :: String -> State -> Int
-labelPercentage l st =
+  -- Show no decimal places if <= 100 successful tests,
+  -- one decimal place if <= 1000 successful tests,
+  -- two decimal places if <= 10000 successful tests, and so on.
+  places :: Integer
+  places =
+    ceiling (logBase 10 (fromIntegral n) - 2 :: Double) `max` 0
+
+labelCount :: String -> State -> Int
+labelCount l st =
   -- XXX in case of a disjunction, a label can occur several times,
   -- need to think what to do there
-  (100 * occur) `div` maxSuccessTests st
-  where
-    occur = length [ l' | l' <- concat (map Set.toList (collected st)), l == l' ]
+  length [ l' | l' <- concat (map Set.toList (collected st)), l == l' ]
 
-insufficientCoverage :: State -> Bool
-insufficientCoverage st =
-  or [ labelPercentage l st < reqP | (l, reqP) <- Map.toList (S.labels st) ]
+percentage :: Integral a => State -> a -> Double
+percentage st n =
+  fromIntegral n * 100 / fromIntegral (numSuccessTests st)
+
+insufficientlyCovered :: State -> [(String, Int, Double)]
+insufficientlyCovered st =
+  [ (l, reqP, p)
+  | (l, reqP) <- Map.toList (S.labels st),
+    let p = percentage st (labelCount l st),
+    p < fromIntegral reqP ]
 
 --------------------------------------------------------------------------
 -- main shrinking loop
 
-foundFailure :: State -> P.Result -> [Rose P.Result] -> IO (Int, Int, Int)
+foundFailure :: State -> P.Result -> [Rose P.Result] -> IO (Int, Int, Int, P.Result)
 foundFailure st res ts =
   do localMin st{ numTryShrinks = 0 } res res ts
 
-localMin :: State -> P.Result -> P.Result -> [Rose P.Result] -> IO (Int, Int, Int)
-localMin st MkResult{P.theException = Just e} lastRes _
-  | isInterrupt e = localMinFound st lastRes
+localMin :: State -> P.Result -> P.Result -> [Rose P.Result] -> IO (Int, Int, Int, P.Result)
+-- Don't try to shrink for too long
+localMin st res _ ts
+  | numSuccessShrinks st + numTotTryShrinks st >= numTotMaxShrinks st =
+    localMinFound st res
 localMin st res _ ts = do
   r <- tryEvaluateIO $
-    putTemp (terminal st)
-      ( short 26 (oneLine (P.reason res))
-     ++ " (after " ++ number (numSuccessTests st+1) "test"
-     ++ concat [ " and "
-              ++ show (numSuccessShrinks st)
-              ++ concat [ "." ++ show (numTryShrinks st) | numTryShrinks st > 0 ]
-              ++ " shrink"
-              ++ (if numSuccessShrinks st == 1
-                  && numTryShrinks st == 0
-                  then "" else "s")
-               | numSuccessShrinks st > 0 || numTryShrinks st > 0
-               ]
-     ++ ")..."
-      )
+    putTemp (terminal st) (failureSummary st res)
   case r of
     Left err ->
       localMinFound st (exception "Exception while printing status message" err) { callbacks = callbacks res }
@@ -398,7 +457,7 @@ localMin st res _ ts = do
             (exception "Exception while generating shrink-list" err) { callbacks = callbacks res }
         Right ts' -> localMin' st res ts'
 
-localMin' :: State -> P.Result -> [Rose P.Result] -> IO (Int, Int, Int)
+localMin' :: State -> P.Result -> [Rose P.Result] -> IO (Int, Int, Int, P.Result)
 localMin' st res [] = localMinFound st res
 localMin' st res (t:ts) =
   do -- CALLBACK before_test
@@ -410,27 +469,13 @@ localMin' st res (t:ts) =
       else localMin st{ numTryShrinks    = numTryShrinks st + 1,
                         numTotTryShrinks = numTotTryShrinks st + 1 } res res ts
 
-localMinFound :: State -> P.Result -> IO (Int, Int, Int)
+localMinFound :: State -> P.Result -> IO (Int, Int, Int, P.Result)
 localMinFound st res =
-  do let report = concat [
-           "(after " ++ number (numSuccessTests st+1) "test",
-           concat [ " and " ++ number (numSuccessShrinks st) "shrink"
-                  | numSuccessShrinks st > 0
-                  ],
-           "): "
-           ]
-     if isOneLine (P.reason res)
-       then putLine (terminal st) (P.reason res ++ " " ++ report)
-       else do
-         putLine (terminal st) report
-         sequence_
-           [ putLine (terminal st) msg
-           | msg <- lines (P.reason res)
-           ]
+  do sequence_ [ putLine (terminal st) msg | msg <- failureReason st res ]
      callbackPostFinalFailure st res
      -- NB no need to check if callbacks threw an exception because
      -- we are about to return to the user anyway
-     return (numSuccessShrinks st, numTotTryShrinks st - numTryShrinks st, numTryShrinks st)
+     return (numSuccessShrinks st, numTotTryShrinks st - numTryShrinks st, numTryShrinks st, res)
 
 --------------------------------------------------------------------------
 -- callbacks
