@@ -25,7 +25,6 @@ module Test.QuickCheck.Text
 --------------------------------------------------------------------------
 -- imports
 
-import Control.Applicative
 import System.IO
   ( hFlush
   , hPutStr
@@ -86,11 +85,11 @@ bold s = s -- for now
 -- putting strings
 
 data Terminal
-  = MkTerminal (IORef String) (IORef Int) (String -> IO ()) (String -> IO ())
+  = MkTerminal (IORef ShowS) (IORef Int) (String -> IO ()) (String -> IO ())
 
 newTerminal :: (String -> IO ()) -> (String -> IO ()) -> IO Terminal
 newTerminal out err =
-  do res <- newIORef ""
+  do res <- newIORef (showString "")
      tmp <- newIORef 0
      return (MkTerminal res tmp out err)
 
@@ -113,25 +112,19 @@ withNullTerminal action =
   newTerminal (const (return ())) (const (return ())) >>= action
 
 terminalOutput :: Terminal -> IO String
-terminalOutput (MkTerminal res _ _ _) = readIORef res
+terminalOutput (MkTerminal res _ _ _) = fmap ($ "") (readIORef res)
 
 handle :: Handle -> String -> IO ()
 handle h s = do
   hPutStr h s
   hFlush h
 
-flush :: Terminal -> IO ()
-flush (MkTerminal _ tmp _ err) =
-  do n <- readIORef tmp
-     writeIORef tmp 0
-     err (replicate n ' ' ++ replicate n '\b')
-
 putPart, putTemp, putLine :: Terminal -> String -> IO ()
 putPart tm@(MkTerminal res _ out _) s =
-  do flush tm
+  do putTemp tm ""
      force s
      out s
-     modifyIORef res (++ s)
+     modifyIORef res (. showString s)
   where
     force :: [a] -> IO ()
     force = evaluate . seqList
@@ -143,9 +136,11 @@ putPart tm@(MkTerminal res _ out _) s =
 putLine tm s = putPart tm (s ++ "\n")
 
 putTemp tm@(MkTerminal _ tmp _ err) s =
-  do flush tm
-     err (s ++ [ '\b' | _ <- s ])
-     modifyIORef tmp (+ length s)
+  do n <- readIORef tmp
+     err $
+       replicate n ' ' ++ replicate n '\b' ++
+       s ++ [ '\b' | _ <- s ]
+     writeIORef tmp (length s)
 
 --------------------------------------------------------------------------
 -- the end.
